import { ChartMeasureRoleType, ChartOrient, getChartCategory, getChartCategory2, getEntityHierarchy } from "@metad/ocap-core"
import { axisOrient, getAxisDataZoom, getCategoryAxis, getMeasureAxis, getValueAxis } from "./axis"
import { AxisEnum, EChartsContext } from "../types"

/**
 * @todo Coordinate system should use x y z identifying axis
 * 
 * @param chartAnnotation
 * @param entityType
 * @param items
 * @param chartOptions
 * @param locale
 * @returns
 */
export function getCoordinateSystem(context: EChartsContext, items: Array<unknown>, locale: string) {
  const { chartAnnotation, entityType, options: chartOptions } = context
  const measure = chartAnnotation.measures.find(
    (item) => !item.role || item.role === ChartMeasureRoleType.Axis1
  )?.measure
  const [categoryOrient, valueOrient] = axisOrient(chartAnnotation.chartType.orient)

  const category = getChartCategory(chartAnnotation)
  const category2 = getChartCategory2(chartAnnotation)

  if (chartAnnotation.chartType.type === 'Scatter') {
    const axis1 = getMeasureAxis(chartAnnotation, ChartMeasureRoleType.Axis1, 0)
    const axis2 = getMeasureAxis(chartAnnotation, ChartMeasureRoleType.Axis2, 1)
    const valueAxises = getValueAxis(chartAnnotation, entityType, chartOptions, locale)
    return {
      categoryAxis: { orient: categoryOrient, axis: [valueAxises[0]], dataZoom: getAxisDataZoom(axis1, AxisEnum.x) },
      valueAxis: { orient: valueOrient, axis: [valueAxises[1]], dataZoom: getAxisDataZoom(axis2, AxisEnum.y) }
    }
  } else {
    if (!category) {
      throw new Error(`Can't find category in chart`)
    }

    let valueAxis = null
    // 设置维度轴值
    const categoryAxis = getCategoryAxis(
      context,
      items,
      category,
      getEntityHierarchy(entityType, category),
      chartOptions,
      measure
    )
    categoryAxis.orient = categoryOrient

    if (category2) {
      valueAxis = [
        getCategoryAxis(context, items, category2, getEntityHierarchy(entityType, category2), chartOptions, measure)
      ]
    } else {
      valueAxis = getValueAxis(chartAnnotation, entityType, chartOptions, locale)
    }
    valueAxis.orient = valueOrient

    return {
      categoryAxis: { orient: categoryOrient, axis: [categoryAxis], dataZoom: getAxisDataZoom(category, categoryOrient) },
      valueAxis: {
        orient: valueOrient,
        axis: valueAxis,
        dataZoom: category2 ? getAxisDataZoom(category2, valueOrient) : getAxisDataZoom(chartAnnotation.measures[0], valueOrient)
      }
    }
  }
}

export function getPolarCoordinateSystem(context: EChartsContext, items: Array<unknown>, locale: string) {
  const { chartAnnotation, entityType, options: chartOptions } = context
  const measure = chartAnnotation.measures.find(
    (item) => !item.role || item.role === ChartMeasureRoleType.Axis1
  )?.measure
  const [categoryOrient, valueOrient] = (() => {
    // Chart Orient
    if (chartAnnotation.chartType.orient === ChartOrient.vertical) {
      return [AxisEnum.angle, AxisEnum.radius]
    } else {
      return [AxisEnum.radius, AxisEnum.angle]
    }
  })()

  const category = getChartCategory(chartAnnotation)
  const category2 = getChartCategory2(chartAnnotation)

  let valueAxis = null
  // 设置维度轴值
  const categoryAxis = getCategoryAxis(
    context,
    items,
    category,
    getEntityHierarchy(entityType, category),
    chartOptions,
    measure
  )
  categoryAxis.orient = categoryOrient

  if (category2) {
    valueAxis = [
      getCategoryAxis(context, items, category2, getEntityHierarchy(entityType, category2), chartOptions, measure)
    ]
  } else {
    valueAxis = getValueAxis(chartAnnotation, entityType, chartOptions, locale)
  }
  valueAxis.orient = valueOrient

  return {
    categoryAxis: { orient: categoryOrient, axis: [categoryAxis], dataZoom: null },
    valueAxis: { orient: valueOrient, axis: valueAxis, dataZoom: null }
  }
}
